"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const core_1 = require("@angular-devkit/core");
const code_utils_1 = require("../utils/code-utils");
const find_module_1 = require("@schematics/angular/utility/find-module");
const parse_name_1 = require("@schematics/angular/utility/parse-name");
const yang_utils_1 = require("../utils/yang-utils");
function default_1(options) {
    return (host) => __awaiter(this, void 0, void 0, function* () {
        var _a, _b, _c, _d, _e, _f;
        const rootPath = yang_utils_1.getSourceRoot(host, options);
        yang_utils_1.smartPath(rootPath, options, 'components');
        if (!options.path) {
            options.path = `${rootPath}/shared/components`;
        }
        const parsedPath = parse_name_1.parseName(options.path, options.name);
        options.path = parsedPath.path;
        options.module = yang_utils_1.findClosestModule(host, options, 'shared');
        const schematic = yield yang_utils_1.getProjectSchematic(host, options, '@schematics/angular:component');
        options.inlineStyle = (_a = options.inlineStyle) !== null && _a !== void 0 ? _a : schematic.inlineStyle;
        options.inlineTemplate = (_b = options.inlineTemplate) !== null && _b !== void 0 ? _b : schematic.inlineTemplate;
        options.flat = (_d = (_c = options.flat) !== null && _c !== void 0 ? _c : schematic.flat) !== null && _d !== void 0 ? _d : false;
        options.skipTests = (_f = (_e = options.skipTests) !== null && _e !== void 0 ? _e : schematic.skipTests) !== null && _f !== void 0 ? _f : false;
        if (!options.entryComponent) {
            delete options.entryComponent;
        }
        const ngOptions = Object.assign(Object.assign({}, options), { skipImport: true });
        return schematics_1.chain([
            schematics_1.externalSchematic('@schematics/angular', 'component', ngOptions),
            addToNgModule(options)
        ]);
    });
}
exports.default = default_1;
function addToNgModule(options) {
    return (host) => {
        if (!options.module || options.skipImport)
            return host;
        const file = options.module;
        // If features module: warn & skip !
        if (file === '/src/app/features/features.module.ts') {
            console.log(yellow('Cannot declare component inside features module: skipping import.'));
            return host;
        }
        const sourceFile = code_utils_1.CodeUtils.readSourceFile(host, file);
        const componentPath = `/${options.path}/`
            + (options.flat ? '' : core_1.strings.dasherize(options.name) + '/')
            + core_1.strings.dasherize(options.name)
            + '.component';
        const relativePath = find_module_1.buildRelativePath(options.module, componentPath);
        code_utils_1.CodeUtils.addImport(sourceFile, `${core_1.strings.classify(options.name)}Component`, relativePath);
        code_utils_1.CodeUtils.insertInVariableArray(sourceFile, "DECLARATIONS", `${core_1.strings.classify(options.name)}Component`);
        code_utils_1.CodeUtils.writeSourceFile(host, file, sourceFile);
        let path = options.path;
        if (path.includes('src/app/features/')) {
            if (options.routing)
                updateFeatureRouting(options, host);
        }
        return host;
    };
}
function updateFeatureRouting(options, host) {
    if (!options.module)
        return;
    // Ajouter la route
    const file = options.module.replace('.module.ts', '-routing.module.ts');
    const sourceFile = code_utils_1.CodeUtils.readSourceFile(host, file);
    if (options.route === undefined) {
        options.route = core_1.strings.dasherize(options.name);
    }
    const componentPath = `/${options.path}/`
        + (options.flat ? '' : core_1.strings.dasherize(options.name) + '/')
        + core_1.strings.dasherize(options.name)
        + '.component';
    const relativePath = find_module_1.buildRelativePath(file, componentPath);
    code_utils_1.CodeUtils.addImport(sourceFile, `${core_1.strings.classify(options.name)}Component`, relativePath);
    code_utils_1.CodeUtils.insertInVariableArray(sourceFile, 'ROUTES', `{ path: '${options.route}', component: ${core_1.strings.classify(options.name)}Component }`);
    code_utils_1.CodeUtils.writeSourceFile(host, file, sourceFile);
}
function yellow(str) {
    return `\x1b[33m${str}\x1b[0m`;
}
//# sourceMappingURL=index.js.map