"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const core_1 = require("@angular-devkit/core");
const yang_utils_1 = require("../utils/yang-utils");
const code_utils_1 = require("../utils/code-utils");
const parse_name_1 = require("@schematics/angular/utility/parse-name");
function default_1(options) {
    return (host) => {
        const rootPath = yang_utils_1.getSourceRoot(host, options);
        if (options.name.includes('/')) {
            let nameArgs = options.name.split('/');
            options.name = nameArgs.pop();
            options.path = nameArgs.join('/');
        }
        options.path = `${rootPath}/features/${options.path || ''}/${core_1.strings.dasherize(options.name)}`;
        const parsedPath = parse_name_1.parseName(options.path, options.name);
        options.path = parsedPath.path;
        options.module = yang_utils_1.findClosestModule(host, options);
        const templateSource = schematics_1.apply(schematics_1.url('./files'), [
            options.skipTests ? schematics_1.noop() : schematics_1.filter(path => !path.endsWith('.spec.ts')),
            schematics_1.template(Object.assign(Object.assign({}, core_1.strings), options)),
            schematics_1.move(options.path)
        ]);
        const createComp = options.component ? schematics_1.schematic('component', {
            project: options.project,
            name: options.name,
            path: options.path,
            routing: true,
            route: '',
            flat: true,
            style: options.style,
            inlineStyle: options.inlineStyle,
            inlineTemplate: options.inlineTemplate
        }) : schematics_1.noop();
        return schematics_1.chain([
            schematics_1.mergeWith(templateSource),
            createComp,
            updateRouting(options)
        ]);
    };
}
exports.default = default_1;
function updateRouting(options) {
    return (host) => {
        if (!options.module)
            return host;
        let file = '';
        let varName = '';
        if (options.module.endsWith(yang_utils_1.YangUtils.FEATURES_MODULE_FILE)) {
            file = options.module;
            varName = 'FEATURES_ROUTES';
        }
        else {
            file = options.module.replace('.module.ts', '-routing.module.ts');
            varName = 'ROUTES';
        }
        // Ajouter la route
        const sourceFile = code_utils_1.CodeUtils.readSourceFile(host, file);
        let routePath = (options.path || '').replace('/src/app/', '@app/');
        routePath = routePath.substring(routePath.indexOf('@app/'));
        code_utils_1.CodeUtils.insertInVariableArray(sourceFile, varName, `{ path: '${core_1.strings.dasherize(options.name)}', loadChildren: () => import('${routePath}/${core_1.strings.dasherize(options.name)}.module').then(m => m.${core_1.strings.classify(options.name)}Module) }`);
        code_utils_1.CodeUtils.writeSourceFile(host, file, sourceFile);
        return host;
    };
}
//# sourceMappingURL=index.js.map