"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const core_1 = require("@angular-devkit/core");
const workspace_1 = require("@schematics/angular/utility/workspace");
const path = require("path");
const os_1 = require("os");
const yang_utils_1 = require("../utils/yang-utils");
const code_utils_1 = require("../utils/code-utils");
const dependencies_1 = require("./dependencies");
const CJSON = require("comment-json");
function default_1(options) {
    return (host) => {
        options.name = options.name || path.basename(process.cwd());
        if (!options.name) {
            throw new schematics_1.SchematicsException(`Invalid options, "name" is required.`);
        }
        const root = yang_utils_1.getProjectRoot(host, options);
        const projectName = yang_utils_1.getProjectName(host, options);
        return schematics_1.chain([
            schematics_1.mergeWith(schematics_1.apply(schematics_1.url('./files/root'), [
                schematics_1.template(Object.assign(Object.assign(Object.assign({}, core_1.strings), options), { buildDate: new Date().toISOString() })),
                schematics_1.move(root),
                yang_utils_1.forceOverwrite(host)
            ]), schematics_1.MergeStrategy.Overwrite),
            updatePackageJson(),
            updateTsConfig(options),
            updateGitIgnore(),
            updatePolyfills(options),
            updateKarmaTest(options),
            updateEnvironments(options),
            updateProjectWorkspace(options),
            schematics_1.schematic('feature', {
                project: projectName,
                name: 'home',
                component: true,
                inlineTemplate: false,
                inlineStyle: false
            }),
            schematics_1.mergeWith(schematics_1.apply(schematics_1.url('./files/home'), [
                schematics_1.template(Object.assign(Object.assign({}, core_1.strings), options)),
                schematics_1.move(path.join(root, 'src/app/features/home')),
            ]), schematics_1.MergeStrategy.Overwrite)
        ]);
    };
}
exports.default = default_1;
function updatePackageJson() {
    return (host) => {
        const pkg = require('../../package.json');
        const filePath = 'package.json';
        if (!host.exists(filePath)) {
            throw new schematics_1.SchematicsException(`File ${filePath} does not exist.`);
        }
        const source = host.read(filePath);
        if (!source) {
            throw new schematics_1.SchematicsException(`File ${filePath} is empty.`);
        }
        const json = CJSON.parse(source.toString('utf-8'));
        json.scripts = Object.assign(Object.assign({}, json.scripts), { 'build': 'ng build --prod' });
        json.dependencies = Object.assign(Object.assign({}, json.dependencies), dependencies_1.extraDependencies);
        json.devDependencies[pkg.name] = `~${pkg.version}`;
        json.devDependencies = Object.assign(Object.assign({}, json.devDependencies), dependencies_1.extraDevDependencies);
        // Sort dependencies by name
        json.dependencies = yang_utils_1.sortByKey(json.dependencies);
        json.devDependencies = yang_utils_1.sortByKey(json.devDependencies);
        host.overwrite(filePath, CJSON.stringify(json, null, 2));
        return host;
    };
}
function updateTsConfig(options) {
    return (host) => {
        const projectRoot = yang_utils_1.getProjectRoot(host, options, true);
        let filePath = 'tsconfig.base.json';
        if (!host.exists(filePath)) {
            filePath = 'tsconfig.json';
        }
        if (!host.exists(filePath)) {
            throw new schematics_1.SchematicsException(`File ${filePath} does not exist.`);
        }
        const source = host.read(filePath);
        if (!source) {
            throw new schematics_1.SchematicsException(`File ${filePath} is empty.`);
        }
        const json = CJSON.parse(source.toString('utf-8'));
        json.compilerOptions = Object.assign(Object.assign({}, json.compilerOptions), { "paths": {
                "@app/*": [projectRoot + 'src/app/*'],
                "@env/*": [projectRoot + 'src/environments/*']
            } });
        host.overwrite(filePath, CJSON.stringify(json, null, 2));
        return host;
    };
}
function updateGitIgnore() {
    return (host) => {
        const filePath = '.gitignore';
        if (!host.exists(filePath))
            return host;
        const source = host.read(filePath);
        if (!source)
            return host;
        let content = source.toString('utf-8');
        content += `${os_1.EOL}# Custom Files${os_1.EOL}`;
        content += `*.iml${os_1.EOL}`;
        content += `/src/assets/app-manifest.json${os_1.EOL}`;
        host.overwrite(filePath, content);
        return host;
    };
}
function updatePolyfills(options) {
    return (host) => {
        const projectRoot = yang_utils_1.getProjectRoot(host, options);
        const filePath = path.join(projectRoot, 'src/polyfills.ts');
        if (!host.exists(filePath))
            return host;
        const source = host.read(filePath);
        if (!source)
            return host;
        let content = source.toString('utf-8');
        content += `import 'whatwg-fetch';${os_1.EOL}`;
        host.overwrite(filePath, content);
        return host;
    };
}
function updateKarmaTest(options) {
    return (host) => {
        const projectRoot = yang_utils_1.getProjectRoot(host, options);
        const file = path.join(projectRoot, 'src/test.ts');
        if (host.exists(file)) {
            const source = host.read(file);
            if (!source)
                return host;
            let content = source.toString('utf-8');
            content += `${os_1.EOL}window[\`APP_MANIFEST\`] = {};${os_1.EOL}`;
            host.overwrite(file, content);
        }
        return host;
    };
}
function updateEnvironments(options) {
    return (host) => {
        const projectRoot = yang_utils_1.getProjectRoot(host, options);
        updateEnvironment(host, path.join(projectRoot, 'src/environments/environment.ts'));
        updateEnvironment(host, path.join(projectRoot, 'src/environments/environment.prod.ts'));
        return host;
    };
}
function updateEnvironment(host, file) {
    if (!host.exists(file))
        return;
    const sourceFile = code_utils_1.CodeUtils.readSourceFile(host, file);
    code_utils_1.CodeUtils.insertInVariableObject(sourceFile, 'environment', `apiUrl: '/api/'`);
    code_utils_1.CodeUtils.writeSourceFile(host, file, sourceFile);
}
function updateProjectWorkspace(options) {
    return (host) => {
        return workspace_1.updateWorkspace(workspace => {
            var _a, _b;
            const projectName = yang_utils_1.getProjectName(host, options);
            const project = workspace.projects.get(projectName);
            if (!project) {
                return;
            }
            workspace.extensions.cli = {
                'defaultCollection': 'yang-schematics'
            };
            let schematics = project.extensions.schematics;
            if (!schematics) {
                schematics = {};
                project.extensions['schematics'] = schematics;
            }
            const componentSchematic = schematics['@schematics/angular:component'];
            componentSchematic.inlineTemplate = (_a = componentSchematic.inlineTemplate) !== null && _a !== void 0 ? _a : false;
            componentSchematic.inlineStyle = (_b = componentSchematic.inlineStyle) !== null && _b !== void 0 ? _b : true;
            schematics['@schematics/angular:component'] = yang_utils_1.sortByKey(componentSchematic);
            const build = project.targets.get('build');
            if (!build) {
                throw new Error(`expected node projects/${projectName}/architect/build in angular.json`);
            }
            const test = project.targets.get('test');
            if (!test) {
                throw new Error(`expected node projects/${projectName}/architect/test in angular.json`);
            }
            const projectRoot = yang_utils_1.getProjectRoot(host, options, true);
            // Add stylePreprocessorOptions
            const stylePreprocessorOptions = {
                "includePaths": [
                    projectRoot + 'src/styles'
                ]
            };
            build.options['stylePreprocessorOptions'] = stylePreprocessorOptions;
            test.options['stylePreprocessorOptions'] = stylePreprocessorOptions;
            // Add @angular-builders/custom-webpack
            build.builder = '@angular-builders/custom-webpack:browser';
            build.options['customWebpackConfig'] = {
                path: projectRoot + 'webpack.config.js'
            };
            const serve = project.targets.get('serve');
            if (!serve) {
                throw new Error(`expected node projects/${projectName}/architect/serve in angular.json`);
            }
            serve.builder = '@angular-builders/custom-webpack:dev-server';
            test.builder = '@angular-builders/custom-webpack:karma';
            test.options['customWebpackConfig'] = {
                path: projectRoot + 'webpack.config.js'
            };
        });
    };
}
//# sourceMappingURL=index.js.map